@@ -7,8 +7,8 @@ Salt compatibility code
 # Import python libs
 from __future__ import absolute_import, unicode_literals, print_function
 import sys
-import types
 import logging
+import binascii
 
 # Import 3rd-party libs
 from salt.exceptions import SaltException
@@ -173,7 +173,7 @@ class IPv6AddressScoped(ipaddress.IPv6Address):
             self._ip = address
         elif self._is_packed_binary(address):
             self._check_packed_address(address, 16)
-            self._ip = ipaddress._int_from_bytes(address, 'big')
+            self._ip = int(binascii.hexlify(address), 16)
         else:
             address = str(address)
             if '/' in address:
@@ -190,7 +190,7 @@ class IPv6AddressScoped(ipaddress.IPv6Address):
         packed = False
         if isinstance(data, bytes) and len(data) == 16 and b':' not in data:
             try:
-                packed = bool(int(str(bytearray(data)).encode('hex'), 16))
+                packed = bool(int(binascii.hexlify(data), 16))
             except ValueError:
                 pass
 
