@@ -398,11 +398,11 @@ def list_nodes(conn=None, call=None):
 
         size = node.get('hardware').get('fixed_instance_size_id', 'Custom size')
 
-        if node.get('private_networks') and len(node['private_networks']) > 0:
+        if node.get('private_networks'):
             for private_ip in node['private_networks']:
                 private_ips.append(private_ip)
 
-        if node.get('ips') and len(node['ips']) > 0:
+        if node.get('ips'):
             for public_ip in node['ips']:
                 public_ips.append(public_ip['ip'])
 
