@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 '''
     Fluent Logging Handler
-    ========================
+    ======================
 
     .. versionadded:: 2015.8.0
 
-    This module provides some `Fluent`_ logging handlers.
+    This module provides some fluentd_ logging handlers.
 
 
     Fluent Logging Handler
-    -------------------
+    ----------------------
 
     In the `fluent` configuration file:
 
@@ -60,16 +60,15 @@
     Log Level
     .........
 
-    The ``fluent_handler``
-    configuration section accepts an additional setting ``log_level``. If not
-    set, the logging level used will be the one defined for ``log_level`` in
-    the global configuration file section.
+    The ``fluent_handler`` configuration section accepts an additional setting
+    ``log_level``. If not set, the logging level used will be the one defined
+    for ``log_level`` in the global configuration file section.
 
     .. admonition:: Inspiration
 
         This work was inspired in `fluent-logger-python`_
 
-    .. _`fluentd`: http://www.fluentd.org
+    .. _fluentd: http://www.fluentd.org
     .. _`fluent-logger-python`: https://github.com/fluent/fluent-logger-python
 
 '''
@@ -166,7 +165,7 @@ def setup_handlers():
         payload_type = __opts__['fluent_handler'].get('payload_type', None)
         # in general, you want the value of tag to ALSO be a member of tags
         tags = __opts__['fluent_handler'].get('tags', ['salt'])
-        tag = tags[0] if len(tags) else 'salt'
+        tag = tags[0] if tags else 'salt'
         if payload_type == 'graylog':
             version = 0
         elif payload_type == 'gelf':
@@ -200,7 +199,7 @@ class MessageFormatter(logging.Formatter, NewStyleClassMixIn):
     def __init__(self, payload_type, version, tags, msg_type=None, msg_path=None):
         self.payload_type = payload_type
         self.version = version
-        self.tag = tags[0] if len(tags) else 'salt'  # 'salt' for backwards compat
+        self.tag = tags[0] if tags else 'salt'  # 'salt' for backwards compat
         self.tags = tags
         self.msg_path = msg_path if msg_path else payload_type
         self.msg_type = msg_type if msg_type else payload_type
