@@ -53,7 +53,6 @@ import time
 
 # Import Salt libs
 from salt.exceptions import SaltInvocationError, CommandExecutionError
-import salt.utils.boto3
 import salt.utils.compat
 import salt.utils.versions
 
@@ -127,7 +126,7 @@ def _delete_resource(name, name_param, desc, res_type, wait=0, status_param=None
     '''
     try:
         wait = int(wait)
-    except:
+    except Exception:
         raise SaltInvocationError("Bad value ('{0}') passed for 'wait' param - must be an "
                                   "int or boolean.".format(wait))
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
@@ -179,7 +178,7 @@ def _create_resource(name, name_param=None, desc=None, res_type=None, wait=0, st
                      **args):
     try:
         wait = int(wait)
-    except:
+    except Exception:
         raise SaltInvocationError("Bad value ('{0}') passed for 'wait' param - must be an "
                                   "int or boolean.".format(wait))
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
@@ -232,7 +231,7 @@ def _modify_resource(name, name_param=None, desc=None, res_type=None, wait=0, st
                      **args):
     try:
         wait = int(wait)
-    except:
+    except Exception:
         raise SaltInvocationError("Bad value ('{0}') passed for 'wait' param - must be an "
                                   "int or boolean.".format(wait))
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
