@@ -74,7 +74,6 @@ except ImportError:
 
 
 # Import Salt libs
-import salt.utils.boto3
 import salt.utils.compat
 import salt.utils.json
 import salt.utils.odict as odict
@@ -499,7 +498,7 @@ def delete(name, force=False, region=None, key=None, keyid=None, profile=None):
 def get_cloud_init_mime(cloud_init):
     '''
     Get a mime multipart encoded string from a cloud-init dict. Currently
-    supports scripts and cloud-config.
+    supports boothooks, scripts and cloud-config.
 
     CLI Example:
 
@@ -829,6 +828,7 @@ def get_instances(name, lifecycle_state="InService", health_status="Healthy",
     while True:
         try:
             asgs = conn.get_all_groups(names=[name])
+            break
         except boto.exception.BotoServerError as e:
             if retries and e.code == 'Throttling':
                 log.debug('Throttled by AWS API, retrying in 5 seconds...')
@@ -885,7 +885,7 @@ def enter_standby(name, instance_ids, should_decrement_desired_capacity=False,
             AutoScalingGroupName=name,
             ShouldDecrementDesiredCapacity=should_decrement_desired_capacity)
     except ClientError as e:
-        err = salt.utils.boto3.get_error(e)
+        err = __utils__['boto3.get_error'](e)
         if e.response.get('Error', {}).get('Code') == 'ResourceNotFoundException':
             return {'exists': False}
         return {'error': err}
@@ -911,7 +911,7 @@ def exit_standby(name, instance_ids, should_decrement_desired_capacity=False,
             InstanceIds=instance_ids,
             AutoScalingGroupName=name)
     except ClientError as e:
-        err = salt.utils.boto3.get_error(e)
+        err = __utils__['boto3.get_error'](e)
         if e.response.get('Error', {}).get('Code') == 'ResourceNotFoundException':
             return {'exists': False}
         return {'error': err}
