@@ -18,7 +18,10 @@ def query(url, **kwargs):
     '''
     Query a resource, and decode the return data
 
-    .. versionadded:: 2015.5.0
+    Passes through all the parameters described in the
+    :py:func:`utils.http.query function <salt.utils.http.query>`:
+
+    .. autofunction:: salt.utils.http.query
 
     CLI Example:
 
