@@ -2,16 +2,20 @@
 '''
 Support for Linux LVM2
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
+import logging
 import os.path
 
 # Import salt libs
-import salt.utils
-import salt.ext.six as six
+import salt.utils.path
+from salt.ext import six
 from salt.exceptions import CommandExecutionError
 
+# Set up logger
+log = logging.getLogger(__name__)
+
 # Define the module's virtual name
 __virtualname__ = 'lvm'
 
@@ -20,7 +24,7 @@ def __virtual__():
     '''
     Only load the module if lvm is installed
     '''
-    if salt.utils.which('lvm'):
+    if salt.utils.path.which('lvm'):
         return __virtualname__
     return (False, 'The linux_lvm execution module cannot be loaded: the lvm binary is not in the path.')
 
@@ -159,7 +163,7 @@ def vgdisplay(vgname=''):
     return ret
 
 
-def lvdisplay(lvname=''):
+def lvdisplay(lvname='', quiet=False):
     '''
     Return information about the logical volume(s)
 
@@ -174,7 +178,10 @@ def lvdisplay(lvname=''):
     cmd = ['lvdisplay', '-c']
     if lvname:
         cmd.append(lvname)
-    cmd_ret = __salt__['cmd.run_all'](cmd, python_shell=False)
+    if quiet:
+        cmd_ret = __salt__['cmd.run_all'](cmd, python_shell=False, output_loglevel='quiet')
+    else:
+        cmd_ret = __salt__['cmd.run_all'](cmd, python_shell=False)
 
     if cmd_ret['retcode'] != 0:
         return {}
@@ -205,7 +212,7 @@ def pvcreate(devices, override=True, **kwargs):
     Set a physical device to be used as an LVM physical volume
 
     override
-        Skip devices, if they are already an LVM physical volumes
+        Skip devices, if they are already LVM physical volumes
 
     CLI Examples:
 
@@ -223,7 +230,6 @@ def pvcreate(devices, override=True, **kwargs):
     for device in devices:
         if not os.path.exists(device):
             raise CommandExecutionError('{0} does not exist'.format(device))
-        # Verify pvcreate was successful
         if not pvdisplay(device):
             cmd.append(device)
         elif not override:
@@ -237,10 +243,13 @@ def pvcreate(devices, override=True, **kwargs):
              'pvmetadatacopies', 'metadatacopies', 'metadataignore',
              'restorefile', 'norestorefile', 'labelsector',
              'setphysicalvolumesize')
+    no_parameter = ('force', 'norestorefile')
     for var in kwargs:
         if kwargs[var] and var in valid:
+            cmd.extend(['--{0}'.format(var), kwargs[var]])
+        elif kwargs[var] and var in no_parameter:
             cmd.append('--{0}'.format(var))
-            cmd.append(kwargs[var])
+
     out = __salt__['cmd.run_all'](cmd, python_shell=False)
     if out.get('retcode'):
         raise CommandExecutionError(out.get('stderr'))
@@ -258,7 +267,7 @@ def pvremove(devices, override=True):
     Remove a physical device being used as an LVM physical volume
 
     override
-        Skip devices, if they are already not used as an LVM physical volumes
+        Skip devices, if they are already not used as LVM physical volumes
 
     CLI Examples:
 
@@ -355,6 +364,7 @@ def lvcreate(lvname,
              pv=None,
              thinvolume=False,
              thinpool=False,
+             force=False,
              **kwargs):
     '''
     Create a new logical volume, with option for which physical volume to be used
@@ -399,7 +409,7 @@ def lvcreate(lvname,
             elif k in valid:
                 extra_arguments.extend(['--{0}'.format(k), '{0}'.format(v)])
 
-    cmd = [salt.utils.which('lvcreate')]
+    cmd = [salt.utils.path.which('lvcreate')]
 
     if thinvolume:
         cmd.extend(['--thin', '-n', lvname])
@@ -429,6 +439,9 @@ def lvcreate(lvname,
     if extra_arguments:
         cmd.extend(extra_arguments)
 
+    if force:
+        cmd.append('--yes')
+
     out = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
     lvdev = '/dev/{0}/{1}'.format(vgname, lvname)
     lvdata = lvdisplay(lvdev)
@@ -467,7 +480,7 @@ def lvremove(lvname, vgname):
     return out.strip()
 
 
-def lvresize(size, lvpath):
+def lvresize(size=None, lvpath=None, extents=None):
     '''
     Return information about the logical volume(s)
 
@@ -477,10 +490,23 @@ def lvresize(size, lvpath):
 
 
         salt '*' lvm.lvresize +12M /dev/mapper/vg1-test
+        salt '*' lvm.lvresize lvpath=/dev/mapper/vg1-test extents=+100%FREE
+
     '''
-    ret = {}
-    cmd = ['lvresize', '-L', str(size), lvpath]
-    cmd_ret = __salt__['cmd.run_all'](cmd, python_shell=False)
-    if cmd_ret['retcode'] != 0:
+    if size and extents:
+        log.error('Error: Please specify only one of size or extents')
         return {}
-    return ret
+
+    cmd = ['lvresize']
+
+    if size:
+        cmd.extend(['-L', '{0}'.format(size)])
+    elif extents:
+        cmd.extend(['-l', '{0}'.format(extents)])
+    else:
+        log.error('Error: Either size or extents must be specified')
+        return {}
+
+    cmd.append(lvpath)
+    cmd_ret = __salt__['cmd.run'](cmd, python_shell=False).splitlines()
+    return {'Output from lvresize': cmd_ret[0].strip()}
