@@ -5,12 +5,15 @@ Module for editing power settings on macOS
  .. versionadded:: 2016.3.0
 '''
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
-# Import salt libs
-import salt.utils
+# Import Salt libs
 import salt.utils.mac_utils
+import salt.utils.platform
 from salt.exceptions import SaltInvocationError
+
+# Import 3rd-party libs
+from salt.ext import six
 from salt.ext.six.moves import range
 
 __virtualname__ = 'power'
@@ -20,7 +23,7 @@ def __virtual__():
     '''
     Only for macOS
     '''
-    if not salt.utils.is_darwin():
+    if not salt.utils.platform.is_darwin():
         return (False, 'The mac_power module could not be loaded: '
                        'module only works on macOS systems.')
 
@@ -38,7 +41,7 @@ def _validate_sleep(minutes):
     Returns: The value to be passed to the command
     '''
     # Must be a value between 1 and 180 or Never/Off
-    if isinstance(minutes, str):
+    if isinstance(minutes, six.string_types):
         if minutes.lower() in ['never', 'off']:
             return 'Never'
         else:
@@ -75,7 +78,8 @@ def get_sleep():
     Computer, Display, and Hard Disk are displayed.
 
     :return: A dictionary containing the sleep status for Computer, Display, and
-    Hard Disk
+        Hard Disk
+
     :rtype: dict
 
     CLI Example:
@@ -284,8 +288,8 @@ def set_wake_on_modem(enabled):
     detected.
 
     :param bool enabled: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
@@ -331,8 +335,8 @@ def set_wake_on_network(enabled):
     is detected.
 
     :param bool enabled: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
@@ -378,8 +382,8 @@ def set_restart_power_failure(enabled):
     failure.
 
     :param bool enabled: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
@@ -427,8 +431,8 @@ def set_restart_freeze(enabled):
     functions remains in case they ever fix the bug.
 
     :param bool enabled: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
@@ -456,7 +460,8 @@ def get_sleep_on_power_button():
     supported
 
     :return: A string value representing the "allow power button to sleep
-    computer" settings
+        computer" settings
+
     :rtype: string
 
     CLI Example:
@@ -476,8 +481,8 @@ def set_sleep_on_power_button(enabled):
     Set whether or not the power button can sleep the computer.
 
     :param bool enabled: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
