@@ -24,6 +24,7 @@ import salt.utils.files
 import salt.utils.path
 import salt.utils.platform
 from salt.exceptions import CommandExecutionError, SaltInvocationError
+from salt.utils.decorators import depends
 
 __virtualname__ = 'system'
 
@@ -255,10 +256,10 @@ def get_system_time(utc_offset=None):
     Get the system time.
 
     :param str utc_offset: The utc offset in 4 digit (+0600) format with an
-    optional sign (+/-).  Will default to None which will use the local
-    timezone. To set the time based off of UTC use "'+0000'". Note: if being
-    passed through the command line will need to be quoted twice to allow
-    negative offsets.
+        optional sign (+/-).  Will default to None which will use the local
+        timezone. To set the time based off of UTC use "'+0000'". Note: if
+        being passed through the command line will need to be quoted twice to
+        allow negative offsets.
     :return: Returns the system time in HH:MM:SS AM/PM format.
     :rtype: str
 
@@ -289,10 +290,10 @@ def set_system_time(newtime, utc_offset=None):
         Meaning you may have to quote the text twice from the command line.
 
     :param str utc_offset: The utc offset in 4 digit (+0600) format with an
-    optional sign (+/-).  Will default to None which will use the local
-    timezone. To set the time based off of UTC use "'+0000'". Note: if being
-    passed through the command line will need to be quoted twice to allow
-    negative offsets.
+        optional sign (+/-).  Will default to None which will use the local
+        timezone. To set the time based off of UTC use "'+0000'". Note: if
+        being passed through the command line will need to be quoted twice to
+        allow negative offsets.
     :return: Returns True if successful. Otherwise False.
     :rtype: bool
 
@@ -316,10 +317,10 @@ def get_system_date_time(utc_offset=None):
     Get the system date/time.
 
     :param str utc_offset: The utc offset in 4 digit (+0600) format with an
-    optional sign (+/-).  Will default to None which will use the local
-    timezone. To set the time based off of UTC use "'+0000'". Note: if being
-    passed through the command line will need to be quoted twice to allow
-    negative offsets.
+        optional sign (+/-).  Will default to None which will use the local
+        timezone. To set the time based off of UTC use "'+0000'". Note: if
+        being passed through the command line will need to be quoted twice to
+        allow negative offsets.
     :return: Returns the system time in YYYY-MM-DD hh:mm:ss format.
     :rtype: str
 
@@ -357,10 +358,10 @@ def set_system_date_time(years=None,
     :param int minutes: Minutes digit: 0 - 59
     :param int seconds: Seconds digit: 0 - 59
     :param str utc_offset: The utc offset in 4 digit (+0600) format with an
-    optional sign (+/-).  Will default to None which will use the local
-    timezone. To set the time based off of UTC use "'+0000'". Note: if being
-    passed through the command line will need to be quoted twice to allow
-    negative offsets.
+        optional sign (+/-).  Will default to None which will use the local
+        timezone. To set the time based off of UTC use "'+0000'". Note: if
+        being passed through the command line will need to be quoted twice to
+        allow negative offsets.
     :return: True if successful. Otherwise False.
     :rtype: bool
 
@@ -409,10 +410,10 @@ def get_system_date(utc_offset=None):
     Get the system date
 
     :param str utc_offset: The utc offset in 4 digit (+0600) format with an
-    optional sign (+/-).  Will default to None which will use the local
-    timezone. To set the time based off of UTC use "'+0000'". Note: if being
-    passed through the command line will need to be quoted twice to allow
-    negative offsets.
+        optional sign (+/-).  Will default to None which will use the local
+        timezone. To set the time based off of UTC use "'+0000'". Note: if
+        being passed through the command line will need to be quoted twice to
+        allow negative offsets.
     :return: Returns the system date.
     :rtype: str
 
@@ -431,7 +432,8 @@ def set_system_date(newdate, utc_offset=None):
     Set the system date. Use <mm-dd-yy> format for the date.
 
     :param str newdate:
-        The date to set. Can be any of the following formats
+        The date to set. Can be any of the following formats:
+
         - YYYY-MM-DD
         - MM-DD-YYYY
         - MM-DD-YY
@@ -464,9 +466,9 @@ def set_system_date(newdate, utc_offset=None):
 # Note that _FixedOffset(0) is a way to build a UTC tzinfo object.
 
 class _FixedOffset(tzinfo):
-    """
+    '''
     Fixed offset in minutes east from UTC.
-    """
+    '''
 
     def __init__(self, offset):
         super(self.__class__, self).__init__()
@@ -608,3 +610,60 @@ def get_computer_name():
         salt '*' network.get_hostname
     '''
     return __salt__['network.get_hostname']()
+
+
+def _is_nilrt_family():
+    '''
+    Determine whether the minion is running on NI Linux RT
+    '''
+    return __grains__.get('os_family') == 'NILinuxRT'
+
+
+NILRT_REBOOT_WITNESS_PATH = '/var/volatile/tmp/salt/reboot_witnessed'
+
+
+@depends('_is_nilrt_family')
+def set_reboot_required_witnessed():
+    '''
+    This function is used to remember that an event indicating that a reboot is
+    required was witnessed. This function writes to a temporary filesystem so
+    the event gets cleared upon reboot.
+
+    Returns:
+        bool: ``True`` if successful, otherwise ``False``
+
+    .. code-block:: bash
+
+        salt '*' system.set_reboot_required_witnessed
+    '''
+    errcode = -1
+    dir_path = os.path.dirname(NILRT_REBOOT_WITNESS_PATH)
+    if not os.path.exists(dir_path):
+        try:
+            os.makedirs(dir_path)
+        except OSError as ex:
+            raise SaltInvocationError('Error creating {0} (-{1}): {2}'
+                                      .format(dir_path, ex.errno, ex.strerror))
+
+        rdict = __salt__['cmd.run_all']('touch {0}'.format(NILRT_REBOOT_WITNESS_PATH))
+        errcode = rdict['retcode']
+
+    return errcode == 0
+
+
+@depends('_is_nilrt_family')
+def get_reboot_required_witnessed():
+    '''
+    Determine if at any time during the current boot session the salt minion
+    witnessed an event indicating that a reboot is required.
+
+    Returns:
+        bool: ``True`` if the a reboot request was witnessed, ``False`` otherwise
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' system.get_reboot_required_witnessed
+    '''
+    return os.path.exists(NILRT_REBOOT_WITNESS_PATH)
