@@ -180,13 +180,14 @@ def _auth(uri):
 
 def extract_war_version(war):
     '''
-    Extract the version from the war file name.  There does not seem to be a
-    standard for encoding the version into the `war file name
-    <https://tomcat.apache.org/tomcat-6.0-doc/deployer-howto.html>`_.
+    Extract the version from the war file name. There does not seem to be a
+    standard for encoding the version into the `war file name`_
+
+    .. _`war file name`: https://tomcat.apache.org/tomcat-6.0-doc/deployer-howto.html
 
     Examples:
 
-    .. code-block:: text
+    .. code-block:: bash
 
         /path/salt-2015.8.6.war -> 2015.8.6
         /path/V6R2013xD5.war -> None
@@ -204,15 +205,18 @@ def _wget(cmd, opts=None, url='http://localhost:8080/manager', timeout=180):
 
     cmd
         the command to execute
+
     url
-        the URL of the server manager webapp
-        example: http://localhost:8080/manager
+        The URL of the server manager webapp (example:
+        http://localhost:8080/manager)
+
     opts
         a dict of arguments
+
     timeout
         timeout for HTTP request
 
-    return value is a dict in the from of::
+    Return value is a dict in the from of::
 
         {
             res: [True|False]
