@@ -20,7 +20,7 @@ import logging
 try:
     import dson
 except ImportError:
-    raise
+    dson = None
 
 # Import salt libs
 from salt.ext import six
@@ -28,6 +28,12 @@ from salt.ext import six
 log = logging.getLogger(__name__)
 
 
+def __virtual__():
+    if dson is None:
+        return False, 'Failed to load: dson module not installed'
+    return True
+
+
 def render(dson_input, saltenv='base', sls='', **kwargs):
     '''
     Accepts DSON data as a string or as a file object and runs it through the
