@@ -4,7 +4,7 @@ Execute salt convenience routines
 '''
 
 # Import python libs
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import, print_function, unicode_literals
 import os
 import logging
 
@@ -12,9 +12,10 @@ import logging
 import salt.exceptions
 import salt.loader
 import salt.minion
-import salt.utils
 import salt.utils.args
 import salt.utils.event
+import salt.utils.files
+import salt.utils.user
 from salt.client import mixins
 from salt.output import display_output
 from salt.utils.lazy import verify_fun
@@ -42,6 +43,7 @@ class RunnerClient(mixins.SyncClientMixin, mixins.AsyncClientMixin, object):
 
     def __init__(self, opts):
         self.opts = opts
+        self.context = {}
 
     @property
     def functions(self):
@@ -50,11 +52,13 @@ class RunnerClient(mixins.SyncClientMixin, mixins.AsyncClientMixin, object):
                 self.utils = salt.loader.utils(self.opts)
             # Must be self.functions for mixin to work correctly :-/
             try:
-                self._functions = salt.loader.runner(self.opts, utils=self.utils)
+                self._functions = salt.loader.runner(
+                    self.opts, utils=self.utils, context=self.context)
             except AttributeError:
                 # Just in case self.utils is still not present (perhaps due to
                 # problems with the loader), load the runner funcs without them
-                self._functions = salt.loader.runner(self.opts)
+                self._functions = salt.loader.runner(
+                    self.opts, context=self.context)
 
         return self._functions
 
@@ -76,23 +80,26 @@ class RunnerClient(mixins.SyncClientMixin, mixins.AsyncClientMixin, object):
             'username', 'password', 'eauth', 'token', 'client', 'user', 'key',
         ] if i in low])
 
-        # Separate the new-style args/kwargs.
-        pre_arg = low.pop('arg', [])
-        pre_kwarg = low.pop('kwarg', {})
-        # Anything not pop'ed from low should hopefully be an old-style kwarg.
-        low['__kwarg__'] = True
-        pre_kwarg.update(low)
+        # Run name=value args through parse_input. We don't need to run kwargs
+        # through because there is no way to send name=value strings in the low
+        # dict other than by including an `arg` array.
+        _arg, _kwarg = salt.utils.args.parse_input(
+                low.pop('arg', []), condition=False)
+        _kwarg.update(low.pop('kwarg', {}))
 
-        # Normalize old- & new-style args in a format suitable for
-        # load_args_and_kwargs
-        old_new_normalized_input = []
-        old_new_normalized_input.extend(pre_arg)
-        old_new_normalized_input.append(pre_kwarg)
+        # If anything hasn't been pop()'ed out of low by this point it must be
+        # an old-style kwarg.
+        _kwarg.update(low)
 
+        # Finally, mung our kwargs to a format suitable for the byzantine
+        # load_args_and_kwargs so that we can introspect the function being
+        # called and fish for invalid kwargs.
+        munged = []
+        munged.extend(_arg)
+        munged.append(dict(__kwarg__=True, **_kwarg))
         arg, kwarg = salt.minion.load_args_and_kwargs(
             self.functions[fun],
-            old_new_normalized_input,
-            self.opts,
+            munged,
             ignore_invalid=True)
 
         return dict(fun=fun, kwarg={'kwarg': kwarg, 'arg': arg},
@@ -118,7 +125,7 @@ class RunnerClient(mixins.SyncClientMixin, mixins.AsyncClientMixin, object):
 
         return mixins.AsyncClientMixin.cmd_async(self, reformatted_low)
 
-    def cmd_sync(self, low, timeout=None):
+    def cmd_sync(self, low, timeout=None, full_return=False):
         '''
         Execute a runner function synchronously; eauth is respected
 
@@ -135,7 +142,7 @@ class RunnerClient(mixins.SyncClientMixin, mixins.AsyncClientMixin, object):
             })
         '''
         reformatted_low = self._reformat_low(low)
-        return mixins.SyncClientMixin.cmd_sync(self, reformatted_low, timeout)
+        return mixins.SyncClientMixin.cmd_sync(self, reformatted_low, timeout, full_return)
 
     def cmd(self, fun, arg=None, pub_data=None, kwarg=None, print_event=True, full_return=False):
         '''
@@ -180,20 +187,26 @@ class Runner(RunnerClient):
         else:
             low = {'fun': self.opts['fun']}
             try:
+                # Allocate a jid
+                async_pub = self._gen_async_pub()
+                self.jid = async_pub['jid']
+
+                fun_args = salt.utils.args.parse_input(
+                        self.opts['arg'],
+                        no_parse=self.opts.get('no_parse', []))
+
                 verify_fun(self.functions, low['fun'])
                 args, kwargs = salt.minion.load_args_and_kwargs(
                     self.functions[low['fun']],
-                    salt.utils.args.parse_input(self.opts['arg']),
-                    self.opts,
-                )
+                    fun_args)
                 low['arg'] = args
                 low['kwarg'] = kwargs
 
                 if self.opts.get('eauth'):
                     if 'token' in self.opts:
                         try:
-                            with salt.utils.fopen(os.path.join(self.opts['cachedir'], '.root_key'), 'r') as fp_:
-                                low['key'] = fp_.readline()
+                            with salt.utils.files.fopen(os.path.join(self.opts['cachedir'], '.root_key'), 'r') as fp_:
+                                low['key'] = salt.utils.stringutils.to_unicode(fp_.readline())
                         except IOError:
                             low['token'] = self.opts['token']
 
@@ -217,13 +230,9 @@ class Runner(RunnerClient):
                         low.update(res)
                         low['eauth'] = self.opts['eauth']
                 else:
-                    user = salt.utils.get_specific_user()
+                    user = salt.utils.user.get_specific_user()
 
-                # Allocate a jid
-                async_pub = self._gen_async_pub()
-                self.jid = async_pub['jid']
-
-                if low['fun'] == 'state.orchestrate':
+                if low['fun'] in ['state.orchestrate', 'state.orch', 'state.sls']:
                     low['kwarg']['orchestration_jid'] = async_pub['jid']
 
                 # Run the runner!
@@ -231,21 +240,23 @@ class Runner(RunnerClient):
                     if self.opts.get('eauth'):
                         async_pub = self.cmd_async(low)
                     else:
-                        async_pub = self.async(self.opts['fun'],
-                                               low,
-                                               user=user,
-                                               pub=async_pub)
+                        async_pub = self.asynchronous(self.opts['fun'],
+                                                      low,
+                                                      user=user,
+                                                      pub=async_pub)
                     # by default: info will be not enougth to be printed out !
-                    log.warning('Running in async mode. Results of this execution may '
-                             'be collected by attaching to the master event bus or '
-                             'by examing the master job cache, if configured. '
-                             'This execution is running under tag {tag}'.format(**async_pub))
+                    log.warning(
+                        'Running in asynchronous mode. Results of this execution may '
+                        'be collected by attaching to the master event bus or '
+                        'by examing the master job cache, if configured. '
+                        'This execution is running under tag %s', async_pub['tag']
+                    )
                     return async_pub['jid']  # return the jid
 
                 # otherwise run it in the main process
                 if self.opts.get('eauth'):
                     ret = self.cmd_sync(low)
-                    if isinstance(ret, dict) and set(ret) == set(('data', 'outputter')):
+                    if isinstance(ret, dict) and set(ret) == {'data', 'outputter'}:
                         outputter = ret['outputter']
                         ret = ret['data']
                     else:
@@ -259,10 +270,27 @@ class Runner(RunnerClient):
                                               async_pub['jid'],
                                               daemonize=False)
             except salt.exceptions.SaltException as exc:
-                ret = '{0}'.format(exc)
+                evt = salt.utils.event.get_event('master', opts=self.opts)
+                evt.fire_event({'success': False,
+                                'return': '{0}'.format(exc),
+                                'retcode': 254,
+                                'fun': self.opts['fun'],
+                                'fun_args': fun_args,
+                                'jid': self.jid},
+                               tag='salt/run/{0}/ret'.format(self.jid))
+                # Attempt to grab documentation
+                if 'fun' in low:
+                    ret = self.get_docs('{0}*'.format(low['fun']))
+                else:
+                    ret = None
+
+                # If we didn't get docs returned then
+                # return the `not availble` message.
+                if not ret:
+                    ret = '{0}'.format(exc)
                 if not self.opts.get('quiet', False):
                     display_output(ret, 'nested', self.opts)
             else:
-                log.debug('Runner return: {0}'.format(ret))
+                log.debug('Runner return: %s', ret)
 
             return ret
