@@ -64,7 +64,7 @@ Setting the grant option is supported as well.
         - object_type: group
         - maintenance_db: testdb
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 
 def __virtual__():
@@ -108,6 +108,9 @@ def present(name,
        - language
        - database
        - group
+       - function
+
+       View permissions should specify `object_type: table`.
 
     privileges
        List of privileges to grant, from the list below:
@@ -228,6 +231,9 @@ def absent(name,
        - language
        - database
        - group
+       - function
+
+       View permissions should specify `object_type: table`.
 
     privileges
        Comma separated list of privileges to revoke, from the list below:
