@@ -29,10 +29,10 @@ import os
 import logging
 
 # Import Salt libs
+import salt.utils.data
 import salt.utils.dateutils
 import salt.utils.platform
 import salt.utils.user
-from salt.utils.locales import sdecode, sdecode_if_string
 from salt.exceptions import CommandExecutionError
 
 # Import 3rd-party libs
@@ -141,13 +141,13 @@ def _changes(name,
             change['empty_password'] = True
         if date is not None and lshad['lstchg'] != date:
             change['date'] = date
-        if mindays and mindays is not 0 and lshad['min'] != mindays:
+        if mindays is not None and lshad['min'] != mindays:
             change['mindays'] = mindays
-        if maxdays and maxdays is not 999999 and lshad['max'] != maxdays:
+        if maxdays is not None and lshad['max'] != maxdays:
             change['maxdays'] = maxdays
-        if inactdays and inactdays is not 0 and lshad['inact'] != inactdays:
+        if inactdays is not None and lshad['inact'] != inactdays:
             change['inactdays'] = inactdays
-        if warndays and warndays is not 7 and lshad['warn'] != warndays:
+        if warndays is not None and lshad['warn'] != warndays:
             change['warndays'] = warndays
         if expire and lshad['expire'] != expire:
             change['expire'] = expire
@@ -156,8 +156,8 @@ def _changes(name,
             change['expire'] = expire
 
     # GECOS fields
-    fullname = sdecode_if_string(fullname)
-    lusr['fullname'] = sdecode_if_string(lusr['fullname'])
+    fullname = salt.utils.data.decode(fullname)
+    lusr['fullname'] = salt.utils.data.decode(lusr['fullname'])
     if fullname is not None and lusr['fullname'] != fullname:
         change['fullname'] = fullname
     if win_homedrive and lusr['homedrive'] != win_homedrive:
@@ -171,24 +171,27 @@ def _changes(name,
 
     # MacOS doesn't have full GECOS support, so check for the "ch" functions
     # and ignore these parameters if these functions do not exist.
-    if 'user.chroomnumber' in __salt__ and roomnumber is not None:
-        roomnumber = sdecode_if_string(roomnumber)
-        lusr['roomnumber'] = sdecode_if_string(lusr['roomnumber'])
+    if 'user.chroomnumber' in __salt__ \
+            and roomnumber is not None:
+        roomnumber = salt.utils.data.decode(roomnumber)
+        lusr['roomnumber'] = salt.utils.data.decode(lusr['roomnumber'])
         if lusr['roomnumber'] != roomnumber:
             change['roomnumber'] = roomnumber
-    if 'user.chworkphone' in __salt__ and workphone is not None:
-        workphone = sdecode_if_string(workphone)
-        lusr['workphone'] = sdecode_if_string(lusr['workphone'])
+    if 'user.chworkphone' in __salt__ \
+            and workphone is not None:
+        workphone = salt.utils.data.decode(workphone)
+        lusr['workphone'] = salt.utils.data.decode(lusr['workphone'])
         if lusr['workphone'] != workphone:
             change['workphone'] = workphone
-    if 'user.chhomephone' in __salt__ and homephone is not None:
-        homephone = sdecode_if_string(homephone)
-        lusr['homephone'] = sdecode_if_string(lusr['homephone'])
+    if 'user.chhomephone' in __salt__ \
+            and homephone is not None:
+        homephone = salt.utils.data.decode(homephone)
+        lusr['homephone'] = salt.utils.data.decode(lusr['homephone'])
         if lusr['homephone'] != homephone:
             change['homephone'] = homephone
     if 'user.chother' in __salt__ and other is not None:
-        other = sdecode_if_string(other)
-        lusr['other'] = sdecode_if_string(lusr['other'])
+        other = salt.utils.data.decode(other)
+        lusr['other'] = salt.utils.data.decode(lusr['other'])
         if lusr['other'] != other:
             change['other'] = other
     # OpenBSD/FreeBSD login class
@@ -467,15 +470,15 @@ def present(name,
             password = __salt__['shadow.gen_password'](password)
 
     if fullname is not None:
-        fullname = sdecode(fullname)
+        fullname = salt.utils.data.decode(fullname)
     if roomnumber is not None:
-        roomnumber = sdecode(roomnumber)
+        roomnumber = salt.utils.data.decode(roomnumber)
     if workphone is not None:
-        workphone = sdecode(workphone)
+        workphone = salt.utils.data.decode(workphone)
     if homephone is not None:
-        homephone = sdecode(homephone)
+        homephone = salt.utils.data.decode(homephone)
     if other is not None:
-        other = sdecode(other)
+        other = salt.utils.data.decode(other)
 
     # createhome not supported on Windows or Mac
     if __grains__['kernel'] in ('Darwin', 'Windows'):
