@@ -8,6 +8,7 @@ from __future__ import absolute_import, print_function, unicode_literals
 import copy
 import fnmatch
 import inspect
+import logging
 import re
 import shlex
 
@@ -20,6 +21,8 @@ import salt.utils.jid
 import salt.utils.versions
 import salt.utils.yaml
 
+log = logging.getLogger(__name__)
+
 
 if six.PY3:
     KWARG_REGEX = re.compile(r'^([^\d\W][\w.-]*)=(?!=)(.*)$', re.UNICODE)
@@ -197,13 +200,16 @@ def yamlify_arg(arg):
 
         elif arg is None \
                 or isinstance(arg, (list, float, six.integer_types, six.string_types)):
-            # yaml.safe_load will load '|' as '', don't let it do that.
-            if arg == '' and original_arg in ('|',):
+            # yaml.safe_load will load '|' and '!' as '', don't let it do that.
+            if arg == '' and original_arg in ('|', '!'):
                 return original_arg
             # yaml.safe_load will treat '#' as a comment, so a value of '#'
             # will become None. Keep this value from being stomped as well.
             elif arg is None and original_arg.strip().startswith('#'):
                 return original_arg
+            # Other times, yaml.safe_load will load '!' as None. Prevent that.
+            elif arg is None and original_arg == '!':
+                return original_arg
             else:
                 return arg
         else:
