@@ -10,6 +10,7 @@ import subprocess
 # Import Salt libs
 import salt.utils.args
 import salt.utils.data
+import salt.utils.winapi
 from salt.ext import six
 from salt.ext.six.moves import range
 from salt.exceptions import CommandExecutionError
@@ -17,7 +18,6 @@ from salt.exceptions import CommandExecutionError
 # Import 3rd-party libs
 try:
     import win32com.client
-    import pythoncom
     import pywintypes
     HAS_PYWIN32 = True
 except ImportError:
@@ -25,6 +25,16 @@ except ImportError:
 
 log = logging.getLogger(__name__)
 
+__virtualname__ = 'win_update'
+
+
+def __virtual__():
+    if not salt.utils.platform.is_windows():
+        return False, 'win_update: Not available on Windows'
+    if not HAS_PYWIN32:
+        return False, 'win_update: Missing pywin32'
+    return __virtualname__
+
 
 class Updates(object):
     '''
@@ -68,7 +78,8 @@ class Updates(object):
         Initialize the updates collection. Can be accessed via
         ``Updates.updates``
         '''
-        self.updates = win32com.client.Dispatch('Microsoft.Update.UpdateColl')
+        with salt.utils.winapi.Com():
+            self.updates = win32com.client.Dispatch('Microsoft.Update.UpdateColl')
 
     def count(self):
         '''
@@ -239,7 +250,8 @@ class WindowsUpdateAgent(object):
     # Error codes found at the following site:
     # https://msdn.microsoft.com/en-us/library/windows/desktop/hh968413(v=vs.85).aspx
     # https://technet.microsoft.com/en-us/library/cc720442(v=ws.10).aspx
-    fail_codes = {-2145124300: 'Download failed: 0x80240034',
+    fail_codes = {-2145107924: 'WinHTTP Send/Receive failed: 0x8024402C',
+                  -2145124300: 'Download failed: 0x80240034',
                   -2145124302: 'Invalid search criteria: 0x80240032',
                   -2145124305: 'Cancelled by policy: 0x8024002F',
                   -2145124307: 'Missing source: 0x8024002D',
@@ -273,13 +285,13 @@ class WindowsUpdateAgent(object):
         Need to look at the possibility of loading this into ``__context__``
         '''
         # Initialize the PyCom system
-        pythoncom.CoInitialize()
+        with salt.utils.winapi.Com():
 
-        # Create a session with the Windows Update Agent
-        self._session = win32com.client.Dispatch('Microsoft.Update.Session')
+            # Create a session with the Windows Update Agent
+            self._session = win32com.client.Dispatch('Microsoft.Update.Session')
 
-        # Create Collection for Updates
-        self._updates = win32com.client.Dispatch('Microsoft.Update.UpdateColl')
+            # Create Collection for Updates
+            self._updates = win32com.client.Dispatch('Microsoft.Update.UpdateColl')
 
         self.refresh()
 
@@ -571,7 +583,8 @@ class WindowsUpdateAgent(object):
         # Initialize the downloader object and list collection
         downloader = self._session.CreateUpdateDownloader()
         self._session.ClientApplicationID = 'Salt: Download Update'
-        download_list = win32com.client.Dispatch('Microsoft.Update.UpdateColl')
+        with salt.utils.winapi.Com():
+            download_list = win32com.client.Dispatch('Microsoft.Update.UpdateColl')
 
         ret = {'Updates': {}}
 
@@ -682,7 +695,8 @@ class WindowsUpdateAgent(object):
 
         installer = self._session.CreateUpdateInstaller()
         self._session.ClientApplicationID = 'Salt: Install Update'
-        install_list = win32com.client.Dispatch('Microsoft.Update.UpdateColl')
+        with salt.utils.winapi.Com():
+            install_list = win32com.client.Dispatch('Microsoft.Update.UpdateColl')
 
         ret = {'Updates': {}}
 
@@ -801,7 +815,8 @@ class WindowsUpdateAgent(object):
 
         installer = self._session.CreateUpdateInstaller()
         self._session.ClientApplicationID = 'Salt: Install Update'
-        uninstall_list = win32com.client.Dispatch('Microsoft.Update.UpdateColl')
+        with salt.utils.winapi.Com():
+            uninstall_list = win32com.client.Dispatch('Microsoft.Update.UpdateColl')
 
         ret = {'Updates': {}}
 
@@ -998,8 +1013,7 @@ def needs_reboot():
 
     '''
     # Initialize the PyCom system
-    pythoncom.CoInitialize()
-
-    # Create an AutoUpdate object
-    obj_sys = win32com.client.Dispatch('Microsoft.Update.SystemInfo')
+    with salt.utils.winapi.Com():
+        # Create an AutoUpdate object
+        obj_sys = win32com.client.Dispatch('Microsoft.Update.SystemInfo')
     return salt.utils.data.is_true(obj_sys.RebootRequired)
